from typing import Any, Mapping, Tuple

from twisted.internet import defer
from twisted.web.http_headers import Headers

import treq

from . import abc as gh_abc


class GitHubAPI(gh_abc.GitHubAPI):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        from twisted.internet import reactor
        self._reactor = reactor
        super().__init__(*args, **kwargs)

    async def _request(self, method: str, url: str, headers: Mapping,
                       body: bytes = b'') -> Tuple[int, Mapping, bytes]:
        # We need to encode the headers to a format that Twisted will like.
        # As a note: treq will set a content-length even if we do, so we need
        # to strip any content-length header.
        headers = Headers(
            {
                k.encode('utf-8'): [v.encode('utf-8')]
                for k, v in headers.items()
                if k.lower() != 'content-length'
            }
        )
        response = await treq.request(method, url, headers=headers, data=body)

        # We need to map the headers back now. In the future, we should fix
        # this up so that any header that appears more than once is handled
        # appropriately.
        response_headers = {
            k.decode('utf-8').lower(): v[0].decode('utf-8')
            for k, v in response.headers.getAllRawHeaders()
        }
        return response.code, response_headers, await response.content()

    async def sleep(self, seconds: float) -> None:
        d = defer.Deferred()
        self._reactor.callLater(seconds, d.callback, None)
        await d



from typing import Mapping, Optional, Tuple

from tornado import gen
from tornado import httpclient

from . import abc as gh_abc


class GitHubAPI(gh_abc.GitHubAPI):

    async def _request(self, method: str, url: str, headers: Mapping,
                       body: bytes = b'') -> Tuple[int, Mapping, bytes]:
        """Make an HTTP request."""
        if method == "GET" and not body:
            real_body = None
        else:
            real_body = body
        request = httpclient.HTTPRequest(url, method, headers, real_body)
        # Since Tornado has designed AsyncHTTPClient to be a singleton, there's
        # no reason not to simply instantiate it every time.
        client = httpclient.AsyncHTTPClient()
        response = await client.fetch(request, raise_error=False)
        return response.code, response.headers, response.body

    async def sleep(self, seconds: float) -> None:
        """Sleep for the specified number of seconds."""
        await gen.sleep(seconds)



import datetime

import pytest
import tornado

from tornado.testing import AsyncTestCase

from .. import BadRequest
from .. import sansio
from .. import tornado as gh_tornado

class TornadoTestCase(AsyncTestCase):

    @tornado.testing.gen_test
    async def test_sleep(self):
        delay = 1
        start = datetime.datetime.now()
        gh = gh_tornado.GitHubAPI("gidgethub")
        await gh.sleep(delay)
        stop = datetime.datetime.now()
        assert (stop - start) > datetime.timedelta(seconds=delay)

    @tornado.testing.gen_test
    async def test__request(self):
        """Make sure that that abstract method is implemented properly."""
        request_headers = sansio.create_headers("gidgethub")
        gh = gh_tornado.GitHubAPI("gidgethub")
        tornado_call = await gh._request("GET", "https://api.github.com/rate_limit",
                                         request_headers)
        data, rate_limit, _ = sansio.decipher_response(*tornado_call)
        assert "rate" in data

    @tornado.testing.gen_test
    async def test__request_with_body(self):
        """Make sure that that abstract method is implemented properly."""
        request_headers = sansio.create_headers("gidgethub")
        gh = gh_tornado.GitHubAPI("gidgethub")
        # This leads to a 404.
        tornado_call = await gh._request("POST", "https://api.github.com/rate_limit",
                                         request_headers, b'bogus')
        with pytest.raises(BadRequest):
            sansio.decipher_response(*tornado_call)

    @tornado.testing.gen_test
    async def test_get(self):
        """Integration test."""
        gh = gh_tornado.GitHubAPI("gidgethub")
        data = await gh.getitem("/rate_limit")
        assert "rate" in data



import datetime

from twisted.internet import reactor
from twisted.internet.defer import ensureDeferred
from twisted.trial.unittest import TestCase
from twisted.web.client import Agent, HTTPConnectionPool

from .. import treq as gh_treq
from .. import sansio

import treq._utils


class TwistedPluginTestCase(TestCase):

    @staticmethod
    def create_cleanup(gh):
        def cleanup(_):
            # We do this just to shut up Twisted.
            pool = treq._utils.get_global_pool()
            pool.closeCachedConnections()

            # We need to sleep to let the connections hang up.
            return ensureDeferred(gh.sleep(0.5))

        return cleanup

    def test_sleep(self):
        delay = 1
        start = datetime.datetime.now()
        gh = gh_treq.GitHubAPI("gidgethub")

        def test_done(ignored):
            stop = datetime.datetime.now()
            self.assertTrue((stop - start) > datetime.timedelta(seconds=delay))
        d = ensureDeferred(gh.sleep(delay))
        d.addCallback(test_done)
        return d

    def test__request(self):
        request_headers = sansio.create_headers("gidgethub")
        gh = gh_treq.GitHubAPI("gidgethub")
        d = ensureDeferred(
            gh._request(
                "GET", "https://api.github.com/rate_limit", request_headers,
            )
        )

        def test_done(response):
            data, rate_limit, _ = sansio.decipher_response(*response)
            self.assertIn("rate", data)

        d.addCallback(test_done)
        d.addCallback(self.create_cleanup(gh))
        return d

    def test_get(self):
        gh = gh_treq.GitHubAPI("gidgethub")
        d = ensureDeferred(gh.getitem("/rate_limit"))

        def test_done(response):
            self.assertIn("rate", response)

        d.addCallback(test_done)
        d.addCallback(self.create_cleanup(gh))
        return d



import asyncio
import datetime
import json
import types

import pytest

from .. import RedirectionException
from .. import abc as gh_abc
from .. import sansio


class MockGitHubAPI(gh_abc.GitHubAPI):

    DEFAULT_HEADERS = {"x-ratelimit-limit": "2", "x-ratelimit-remaining": "1",
                       "x-ratelimit-reset": "0",
                       "content-type": "application/json"}

    def __init__(self, status_code=200, headers=DEFAULT_HEADERS, body=b'', *,
                 cache=None):
        self.response_code = status_code
        self.response_headers = headers
        self.response_body = body
        super().__init__("test_abc", oauth_token="oauth token", cache=cache)

    async def _request(self, method, url, headers, body=b''):
        """Make an HTTP request."""
        self.method = method
        self.url = url
        self.headers = headers
        self.body = body
        response_headers = self.response_headers.copy()
        try:
            # Don't loop forever.
            del self.response_headers["link"]
        except KeyError:
            pass
        return self.response_code, response_headers, self.response_body

    async def sleep(self, seconds):  # pragma: no cover
        """Sleep for the specified number of seconds."""
        self.slept = seconds


@pytest.mark.asyncio
async def test_url_formatted():
    """The URL is appropriately formatted."""
    gh = MockGitHubAPI()
    await gh._make_request("GET", "/users/octocat/following{/other_user}",
                           {"other_user": "brettcannon"}, "",
                           sansio.accept_format())
    assert gh.url == "https://api.github.com/users/octocat/following/brettcannon"


@pytest.mark.asyncio
async def test_headers():
    """Appropriate headers are created."""
    accept = sansio.accept_format()
    gh = MockGitHubAPI()
    await gh._make_request("GET", "/rate_limit", {}, "", accept)
    assert gh.headers["user-agent"] == "test_abc"
    assert gh.headers["accept"] == accept
    assert gh.headers["authorization"] == "token oauth token"


@pytest.mark.asyncio
async def test_rate_limit_set():
    """The rate limit is updated after receiving a response."""
    rate_headers = {"x-ratelimit-limit": "42", "x-ratelimit-remaining": "1",
                    "x-ratelimit-reset": "0"}
    gh = MockGitHubAPI(headers=rate_headers)
    await gh._make_request("GET", "/rate_limit", {}, "", sansio.accept_format())
    assert gh.rate_limit.limit == 42


@pytest.mark.asyncio
async def test_decoding():
    """Test that appropriate decoding occurs."""
    original_data = {"hello": "world"}
    headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
    headers['content-type'] = "application/json; charset=utf-8"
    gh = MockGitHubAPI(headers=headers,
                       body=json.dumps(original_data).encode("utf8"))
    data, _ = await gh._make_request("GET", "/rate_limit", {}, '',
                                          sansio.accept_format())
    assert data == original_data


@pytest.mark.asyncio
async def test_more():
    """The 'next' link is returned appropriately."""
    headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
    headers['link'] = ("<https://api.github.com/fake?page=2>; "
                       "rel=\"next\"")
    gh = MockGitHubAPI(headers=headers)
    _, more = await gh._make_request("GET", "/fake", {}, "",
                                          sansio.accept_format())
    assert more == "https://api.github.com/fake?page=2"


@pytest.mark.asyncio
async def test_getitem():
    original_data = {"hello": "world"}
    headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
    headers['content-type'] = "application/json; charset=UTF-8"
    gh = MockGitHubAPI(headers=headers,
                       body=json.dumps(original_data).encode("utf8"))
    data = await gh.getitem("/fake")
    assert gh.method == "GET"
    assert data == original_data


@pytest.mark.asyncio
async def test_getiter():
    """Test that getiter() returns an async iterable as well as URI expansion."""
    original_data = [1, 2]
    next_url = "https://api.github.com/fake{/extra}?page=2"
    headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
    headers['content-type'] = "application/json; charset=UTF-8"
    headers["link"] = f'<{next_url}>; rel="next"'
    gh = MockGitHubAPI(headers=headers,
                       body=json.dumps(original_data).encode("utf8"))
    data = []
    async for item in gh.getiter("/fake", {"extra": "stuff"}):
        data.append(item)
    assert gh.method == "GET"
    assert gh.url == "https://api.github.com/fake/stuff?page=2"
    assert len(data) == 4
    assert data[0] == 1
    assert data[1] == 2
    assert data[2] == 1
    assert data[3] == 2


@pytest.mark.asyncio
async def test_post():
    send = [1, 2, 3]
    send_json = json.dumps(send).encode("utf-8")
    receive = {"hello": "world"}
    headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
    headers['content-type'] = "application/json; charset=utf-8"
    gh = MockGitHubAPI(headers=headers,
                       body=json.dumps(receive).encode("utf-8"))
    data = await gh.post("/fake", data=send)
    assert gh.method == "POST"
    assert gh.headers['content-type'] == "application/json; charset=utf-8"
    assert gh.body == send_json
    assert gh.headers['content-length'] == str(len(send_json))


@pytest.mark.asyncio
async def test_patch():
    send = [1, 2, 3]
    send_json = json.dumps(send).encode("utf-8")
    receive = {"hello": "world"}
    headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
    headers['content-type'] = "application/json; charset=utf-8"
    gh = MockGitHubAPI(headers=headers,
                       body=json.dumps(receive).encode("utf-8"))
    data = await gh.patch("/fake", data=send)
    assert gh.method == "PATCH"
    assert gh.headers['content-type'] == "application/json; charset=utf-8"
    assert gh.body == send_json
    assert gh.headers['content-length'] == str(len(send_json))


@pytest.mark.asyncio
async def test_put():
    send = [1, 2, 3]
    send_json = json.dumps(send).encode("utf-8")
    receive = {"hello": "world"}
    headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
    headers['content-type'] = "application/json; charset=utf-8"
    gh = MockGitHubAPI(headers=headers,
                       body=json.dumps(receive).encode("utf-8"))
    data = await gh.put("/fake", data=send)
    assert gh.method == "PUT"
    assert gh.headers['content-type'] == "application/json; charset=utf-8"
    assert gh.body == send_json
    assert gh.headers['content-length'] == str(len(send_json))


@pytest.mark.asyncio
async def test_delete():
    send = [1, 2, 3]
    send_json = json.dumps(send).encode("utf-8")
    receive = {"hello": "world"}
    headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
    headers['content-type'] = "application/json; charset=utf-8"
    gh = MockGitHubAPI(headers=headers,
                       body=json.dumps(receive).encode("utf-8"))
    data = await gh.delete("/fake", data=send)
    assert gh.method == "DELETE"
    assert gh.headers['content-type'] == "application/json; charset=utf-8"
    assert gh.body == send_json
    assert gh.headers['content-length'] == str(len(send_json))


class TestCache:

    @pytest.mark.asyncio
    async def test_if_none_match_sent(self):
        etag = "12345"
        cache = {"https://api.github.com/fake": (etag, None, "hi", None)}
        gh = MockGitHubAPI(cache=cache)
        await gh.getitem("/fake")
        assert "if-none-match" in gh.headers
        assert gh.headers["if-none-match"] == etag


    @pytest.mark.asyncio
    async def test_etag_received(self):
        cache = {}
        etag = "12345"
        headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
        headers["etag"] = etag
        gh = MockGitHubAPI(200, headers, b'42', cache=cache)
        data = await gh.getitem("/fake")
        url = "https://api.github.com/fake"
        assert url in cache
        assert cache[url] == (etag, None, 42, None)
        assert data == cache[url][2]

    @pytest.mark.asyncio
    async def test_if_modified_since_sent(self):
        last_modified = "12345"
        cache = {"https://api.github.com/fake": (None, last_modified, "hi", None)}
        gh = MockGitHubAPI(cache=cache)
        await gh.getitem("/fake")
        assert "if-modified-since" in gh.headers
        assert gh.headers["if-modified-since"] == last_modified

    @pytest.mark.asyncio
    async def test_last_modified_received(self):
        cache = {}
        last_modified = "12345"
        headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
        headers["last-modified"] = last_modified
        gh = MockGitHubAPI(200, headers, b'42', cache=cache)
        data = await gh.getitem("/fake")
        url = "https://api.github.com/fake"
        assert url in cache
        assert cache[url] == (None, last_modified, 42, None)
        assert data == cache[url][2]

    @pytest.mark.asyncio
    async def test_hit(self):
        url = "https://api.github.com/fake"
        cache = {url: ("12345", "67890", 42, None)}
        gh = MockGitHubAPI(304, cache=cache)
        data = await gh.getitem(url)
        assert data == 42

    @pytest.mark.asyncio
    async def test_miss(self):
        url = "https://api.github.com/fake"
        cache = {url: ("12345", "67890", 42, None)}
        headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
        headers["etag"] = "09876"
        headers["last-modified"] = "54321"
        gh = MockGitHubAPI(200, headers, body=b"-13", cache=cache)
        data = await gh.getitem(url)
        assert data == -13
        assert cache[url] == ("09876", "54321", -13, None)

    @pytest.mark.asyncio
    async def test_ineligible(self):
        cache = {}
        gh = MockGitHubAPI(cache=cache)
        url = "https://api.github.com/fake"
        # Only way to force a GET request with a body.
        await gh._make_request("GET", url, {}, 42, "asdf")
        assert url not in cache
        await gh.post(url, data=42)
        assert url not in cache

    @pytest.mark.asyncio
    async def test_redirect_without_cache(self):
        cache = {}
        gh = MockGitHubAPI(304, cache=cache)
        with pytest.raises(RedirectionException):
            await gh.getitem("/fake")

    @pytest.mark.asyncio
    async def test_no_cache(self):
        headers = MockGitHubAPI.DEFAULT_HEADERS.copy()
        headers["etag"] = "09876"
        headers["last-modified"] = "54321"
        gh = MockGitHubAPI(headers=headers)
        await gh.getitem("/fake")  # No exceptions raised.



"""An async GitHub API library"""
__version__ = '2.5.0.dev'

import http
from typing import Any


class GitHubException(Exception):

    """Base exception for this library."""


class ValidationFailure(GitHubException):

    """An exception representing failed validation of a webhook event."""
    # https://developer.github.com/webhooks/securing/#validating-payloads-from-github


class HTTPException(GitHubException):

    """A general exception to represent HTTP responses."""

    def __init__(self, status_code: http.HTTPStatus, *args: Any) -> None:
        self.status_code = status_code
        if args:
            super().__init__(*args)
        else:
            super().__init__(status_code.phrase)


class RedirectionException(HTTPException):

    """Exception for 3XX HTTP responses."""


class BadRequest(HTTPException):
    """The request is invalid.

    Used for 4XX HTTP errors.
    """
    # https://developer.github.com/v3/#client-errors


class RateLimitExceeded(BadRequest):

    """Request rejected due to the rate limit being exceeded."""

    # Technically rate_limit is of type gidgethub.sansio.RateLimit, but a
    # circular import comes about if you try to properly declare it.
    def __init__(self, rate_limit: Any, *args: Any) -> None:
        self.rate_limit = rate_limit

        if not args:
            super().__init__(http.HTTPStatus.FORBIDDEN,
                             "rate limit exceeded")
        else:
            super().__init__(http.HTTPStatus.FORBIDDEN, *args)


class InvalidField(BadRequest):

    """A field in the request is invalid.

    Represented by a 422 HTTP Response. Details of what fields were
    invalid are stored in the errors attribute.
    """

    def __init__(self, errors: Any, *args: Any) -> None:
        """Store the error details."""
        self.errors = errors
        super().__init__(http.HTTPStatus.UNPROCESSABLE_ENTITY, *args)


class GitHubBroken(HTTPException):

    """Exception for 5XX HTTP responses."""



import http

from .. import (BadRequest, GitHubBroken, HTTPException, InvalidField,
                RateLimitExceeded, RedirectionException)
from .. import sansio


class TestHTTPException:

    def test_status_code_only(self):
        """The status is the message if a better one isn't provided."""
        exc = HTTPException(http.HTTPStatus.BAD_REQUEST)
        assert exc.status_code == http.HTTPStatus.BAD_REQUEST
        assert str(exc) == "Bad Request"

    def test_message(self):
        """A provided message overrides the status code as the message."""
        message = "a better message"
        exc = HTTPException(http.HTTPStatus.BAD_REQUEST, message)
        assert exc.status_code == http.HTTPStatus.BAD_REQUEST
        assert str(exc) == message


def test_GitHubBroken():
    exc = GitHubBroken(http.HTTPStatus.BAD_GATEWAY)
    assert exc.status_code == http.HTTPStatus.BAD_GATEWAY

def test_BadRequest():
    exc = BadRequest(http.HTTPStatus.BAD_REQUEST)
    assert exc.status_code == http.HTTPStatus.BAD_REQUEST

def test_RateLimitExceeded():
    rate = sansio.RateLimit(limit=1, remaining=0, reset_epoch=1)
    exc = RateLimitExceeded(rate)
    assert exc.status_code == http.HTTPStatus.FORBIDDEN
    exc = RateLimitExceeded(rate, "stuff happened")
    assert str(exc) == "stuff happened"

def test_InvalidField():
    errors = [{"resource": "Issue", "field": "title", "code": "missing_field"}]
    exc = InvalidField(errors, "Validation Failed")
    assert exc.errors == errors
    assert exc.status_code == http.HTTPStatus.UNPROCESSABLE_ENTITY

def test_RedirectionException():
    exc = RedirectionException(http.HTTPStatus.MOVED_PERMANENTLY)
    assert exc.status_code == http.HTTPStatus.MOVED_PERMANENTLY



"""Provide an abstract base class for easier requests."""
import abc
import json
from typing import Any, AsyncGenerator, Dict, Mapping, MutableMapping, Tuple
from typing import Optional as Opt

from . import sansio


# Value represents etag, last-modified, data, and next page.
CACHE_TYPE = MutableMapping[str, Tuple[Opt[str], Opt[str], Any, Opt[str]]]


class GitHubAPI(abc.ABC):

    """Provide an idiomatic API for making calls to GitHub's API."""

    def __init__(self, requester: str, *, oauth_token: Opt[str] = None,
                 cache: Opt[CACHE_TYPE] = None) -> None:
        self.requester = requester
        self.oauth_token = oauth_token
        self._cache = cache
        self.rate_limit: Opt[sansio.RateLimit] = None

    @abc.abstractmethod
    async def _request(self, method: str, url: str, headers: Mapping,
                       body: bytes = b'') -> Tuple[int, Mapping, bytes]:
        """Make an HTTP request."""

    @abc.abstractmethod
    async def sleep(self, seconds: float) -> None:
        """Sleep for the specified number of seconds."""

    async def _make_request(self, method: str, url: str, url_vars: Dict,
                            data: Any, accept: str) -> Tuple[bytes, Opt[str]]:
        """Construct and make an HTTP request."""
        filled_url = sansio.format_url(url, url_vars)
        request_headers = sansio.create_headers(self.requester, accept=accept,
                                                oauth_token=self.oauth_token)
        cached = cacheable = False
        # Can't use None as a "no body" sentinel as it's a legitimate JSON type.
        if data == b"":
            body = b""
            request_headers["content-length"] = "0"
            if method == "GET" and self._cache is not None:
                cacheable = True
                try:
                    etag, last_modified, data, more = self._cache[filled_url]
                    cached = True
                except KeyError:
                    pass
                else:
                    if etag is not None:
                        request_headers["if-none-match"] = etag
                    if last_modified is not None:
                        request_headers["if-modified-since"] = last_modified
        else:
            charset = "utf-8"
            body = json.dumps(data).encode(charset)
            request_headers['content-type'] = f"application/json; charset={charset}"
            request_headers['content-length'] = str(len(body))
        if self.rate_limit is not None:
            self.rate_limit.remaining -= 1
        response = await self._request(method, filled_url, request_headers, body)
        if not (response[0] == 304 and cached):
            data, self.rate_limit, more = sansio.decipher_response(*response)
            has_cache_details = ("etag" in response[1]
                                 or "last-modified" in response[1])
            if self._cache is not None and cacheable and has_cache_details:
                etag = response[1].get("etag")
                last_modified = response[1].get("last-modified")
                self._cache[filled_url] = etag, last_modified, data, more
        return data, more

    async def getitem(self, url: str, url_vars: Dict = {},
                      *, accept: str = sansio.accept_format()) -> Any:
        """Send a GET request for a single item to the specified endpoint."""
        data, _ = await self._make_request("GET", url, url_vars, b"", accept)
        return data

    async def getiter(self, url: str, url_vars: Dict = {},
                      *, accept: str = sansio.accept_format()) -> AsyncGenerator[Any, None]:
        """Return an async iterable for all the items at a specified endpoint."""
        data, more = await self._make_request("GET", url, url_vars, b"", accept)
        for item in data:
            yield item
        if more:
            # `yield from` is not supported in coroutines.
            async for item in self.getiter(more, url_vars, accept=accept):
                yield item

    async def post(self, url: str, url_vars: Dict = {}, *, data: Any,
                   accept: str = sansio.accept_format()) -> Any:
        data, _ = await self._make_request("POST", url, url_vars, data, accept)
        return data

    async def patch(self, url: str, url_vars: Dict = {}, *, data: Any,
                    accept: str = sansio.accept_format()) -> Any:
        data, _ = await self._make_request("PATCH", url, url_vars, data, accept)
        return data

    async def put(self, url: str, url_vars: Dict = {}, *, data: Any = b"",
                  accept: str = sansio.accept_format()) -> Any:
        data, _ = await self._make_request("PUT", url, url_vars, data, accept)
        return data

    async def delete(self, url: str, url_vars: Dict = {}, *, data: Any = b"",
                     accept: str = sansio.accept_format()) -> None:
        await self._make_request("DELETE", url, url_vars, data, accept)



import asyncio
from typing import Any, Mapping, Tuple

import aiohttp

from . import abc as gh_abc


class GitHubAPI(gh_abc.GitHubAPI):

    def __init__(self, session: aiohttp.ClientSession, *args: Any,
                 **kwargs: Any) -> None:
        self._session = session
        super().__init__(*args, **kwargs)

    async def _request(self, method: str, url: str, headers: Mapping,
                       body: bytes = b'') -> Tuple[int, Mapping, bytes]:
        async with self._session.request(method, url, headers=headers,
                                         data=body) as response:
            return response.status, response.headers, await response.read()

    async def sleep(self, seconds: float) -> None:
        await asyncio.sleep(seconds)



import pathlib

import setuptools


docs_requires = ["sphinx"]
tests_requires = ['pytest>=3.0.0', 'pytest-asyncio']
aiohttp_requires = ["aiohttp"]
treq_requires = ["treq", "twisted[tls]"]
tornado_requires = ["tornado"]

long_description = pathlib.Path("README.rst").read_text("utf-8")

setuptools.setup(
    name="gidgethub",
    version="2.5.0-post1",
    description="An async GitHub API library",
    long_description=long_description,
    url="https://gidgethub.readthedocs.io",
    author="Brett Cannon",
    author_email="brett@python.org",
    license="Apache",
    classifiers=[
        'Intended Audience :: Developers',
        "License :: OSI Approved :: Apache Software License",
        "Development Status :: 5 - Production/Stable",
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
    ],
    keywords="github sans-io async",
    packages=setuptools.find_packages(),
    zip_safe=True,
    python_requires=">=3.6.0",
    setup_requires=['pytest-runner>=2.11.0'],
    tests_require=tests_requires,
    install_requires=['uritemplate>=3.0.0'],
    extras_require={
        "docs": docs_requires,
        "tests": tests_requires,
        "aiohttp": aiohttp_requires,
        "treq": treq_requires,
        "tornado": tornado_requires,
        "dev": (docs_requires + tests_requires + aiohttp_requires
                + treq_requires + tornado_requires),
    },
)
